
"""
analyze script for KNITRO, also callable from system
"""

import sys
import os

def analyze(argv):
    """
    analyze script for KNITRO, arguments:
    <sourcedir> <targetdir> <plistfile> <solverdir> <languagedir> <timeout> 
    """
    scriptname=argv[0]
    sourcedir=argv[1]
    targetdir=argv[2]
    plistfile=argv[3]

    try:
        f=open(os.path.join(sourcedir,plistfile),'r')
    except IOError, errno:
        print errno
    lines=f.readlines()
    f.close()
    for i in lines:
        if i.find('.lst')!=-1:
            filein=i.replace('\n','')
            fileout=os.path.join(targetdir,os.path.basename(i).replace('.lst','.res'))
            fileout=fileout.replace('\n','')

            f1=open(filein,'r')
            lines1=f1.readlines()
            f1.close()

            f2=open(fileout,'w')

            modelstatus=5
            x=[]
            xfound=0
            objstr='0'
            infeasstr='0'
            timestr='0'
            for j in lines1:
                # determine model status:
                if j.find('EXIT: ')!=-1:
                    if str.lower(j)=='exit: convergence to an infeasible point. problem appears to be locally\n':
                        modelstatus=2
                    elif str.lower(j)=='exit: convergence to an infeasible point.\n':
                        modelstatus=2
                    elif str.lower(j)=='exit: terminate at infeasible point because the relative change in solution\n':
                        modelstatus=2
                    elif str.lower(j)=='exit: time limit reached.\n':
                        modelstatus=-2
                    elif str.lower(j)=='exit: iteration limit reached.\n':
                        modelstatus=-2
                    elif str.lower(j)=='exit: locally optimal solution found.\n':
                        modelstatus=1
                    elif str.lower(j)=='exit: optimal solution found.\n':
                        modelstatus=1
                    elif str.lower(j)=='exit: evaluation error.\n':
                        modelstatus=2
                    elif str.lower(j)=='exit: relative change in solution estimate < xtol.\n':
                        modelstatus=2
                # find solution strings:
                if j.find('x[')!=-1:
                    eqsplit=j.split('=')[1].replace(' ','')
                    eqsplit=eqsplit.replace('\n','')
                    x.append(eqsplit)
                    xfound=1
                # find objective function value:
                if j.find('Final objective value ')!=-1:
                    eqsplit=j.split('=')[1].replace(' ','')
                    objstr=eqsplit.replace('\n','')
                # find feasibility:
                if j.find('Final feasibility error ')!=-1:
                    eqsplit=j.split('=')[1].replace('\n','')
                    infeasstr=eqsplit.split('/')[0].replace(' ','')
                # find time:
                if j.find('Total program time ')!=-1:
                    eqsplit=j.split('=')[1].replace('\n','')
                    timestr=eqsplit.split('(')[1].split('CPU')[0].replace(' ','')
                # account for anomalies:
                if j.find('No variables declared.')!=-1:
                    modelstatus=3
                    x.append('0')
                    xfound=1
                if str.lower(j)=='error: unbounded constraint specification.\n':
                    modelstatus=3
                    x.append('0')

                    xfound=1

            if xfound==0:
                modelstatus=3
                x.append('0')

            f2.write('modelstatus = ')
            f2.write(str(modelstatus))
            f2.write('\n')

            count=0
            for j in x:
                count=count+1
                f2.write('x(')
                f2.write(str(count))
                f2.write(') = ')
                f2.write(j)
                f2.write('\n')

            f2.write('obj = ')
            f2.write(objstr)
            f2.write('\n')

            f2.write('infeas = ')
            f2.write(infeasstr)
            f2.write('\n')

            if (modelstatus==0) | (modelstatus==1):
                f2.write('nonopt = 0\n')
            else:
                f2.write('nonopt = 1\n')

            f2.write('time = ')
            f2.write(timestr)
            f2.write('\n')
            
            f2.close()

if __name__=='__main__':
  analyze(sys.argv)
